unit ItsyBits;

{
  Unit:               ITSYBITS.PAS
  Description:        Provides support for windows with small captions.
  Version:            1.0
  Creation Date:      January 7, 1993
  Modification Date:  January 12, 1993
  Operating System:   MS-DOS 3.x and Windows 3.1
  Hardware Required:  Windows-capable computer system
  Programming System: Borland Pascal for Windows 7.0
  BPW Translation by: Craig Boyd
  Original Author:    Charles E. Kindel, Jr. (9/27/91).


  About This Unit

  This is a BPW translation of ITSYBITS.C, which can be found in
  ITSBITS.ZIP in Lib 5 of the WINSDK forum on CompuServe.  Consult
  ITSYBITS.DOC (included in ITSBITS.ZIP) for more information on using
  ItsyBitsy windows.  Note that there is another version of ITSBITS.ZIP
  in Lib 1 of WINSDK that has a later upload date.  I haven't looked at
  it, so I don't know what, if anything, is different from the version
  in Lib 5.  Note also that forum libraries can be rearranged at a
  moment's notice, so please don't blame me if ITSBITS.ZIP turns up
  missing!

  The only thing I've added to ITSYBITS.PAS that's not in ITSYBITS.C is
  the TItsyBitsWindow object declaration, which let's you use ItsyBits
  windows in OWL-based programs.  IBTEST.PAS is a sample program
  showing how to use windows derived from TIstyBitsWindow.

  The following comments are from the original author:

    ItsyBitsy is a support module that allows you to create windows that
    look and act very much like a popup window with a system menu and
    caption bar, except everything is scaled to about 2/3 scale.

    For documentation on how to use ItsyBits, read the document
    ITSYBITS.RTF, which is in rich text format.

    Little known fact:  ExtTextOut() is the fastest way to draw a filled
    rectangle in Windows (if you don't want dithered colors or borders).

    Unfortunately there is a bug in the Windows 3.0 8514 driver in using
    ExtTextOut() to a memory DC.  If you are drawing to an off screen
    bitmap, then blitting that bitmap to the display, do not define
    USE_EXTTEXTOUT below.

    The macro (DRAWFASTRECT) draws a filled rectangle with no border and a
    solid color.  It uses the current background color as the fill color.


  Update History

  update    ver   description (author)
  -------   ---   -----------
  9301.07   0.0   Work begun on translating C source code. (CSB)
  9301.08   1.0   First working version. (CSB)
  9301.12   1.0   Updated comments. (CSB)
}

{$define USE_EXTTEXTOUT}

{-- Interface Section ---------------------------------------------------}

{$R-,S-}

interface

uses
  WinTypes,
  OWindows;

const
  ibs_HorzCaption = $00004000;
  ibs_VertCaption = $00008000;

function ibDefWindowProc(HWin   : hwnd;
                         wMsg,
                         wParam : word;
                         lParam : longint) : longint;
{
  This function should be called instead of DefWindowProc() for windows
  that want to have itsybitsy captions.
}

procedure ibAdjustWindowRect(    HWin : hwnd;
                             var lprc : TRect);
{
  Does the same thing as the USER function AdjustWindowRect(), but knows
  about itsybitsy windows.  AdjustWindowRect() is bogus it appears.
}

type
  PItsyBitsWindow = ^TItsyBitsWindow;
  TItsyBitsWindow = object(TWindow)
    constructor Init(aParent: PWindowsObject; aTitle: PChar; ibs_Style : longint);
    procedure DefWndProc(var Msg : TMessage); virtual;
    function GetClassName : pchar; virtual;
  end;

{-- Implementation Section ----------------------------------------------}

implementation

uses
  WinProcs,
  Win31;

const
              {  BBGGRR}
  RGBLtGray = $00C0C0C0;
  RGBGray   = $00808080;
  RGBBlack  = $00000000;
  RGBWhite  = $00FFFFFF;

var
  gcCaption : word;
  fWin31    : bool;

{-- TItsyBits Methods ---------------------------------------------------}

constructor TItsyBitsWindow.Init;
{
  Overrides TWindow.Init.  Attr.Style is set to (ws_ThickFrame or
  ws_Border or ws_Popup or ws_SysMenu or ws_Visible).  Attr.Style also
  includes the ibs_Style bit, which determines what type of caption this
  ItsyBits window will have.  The allowable values are:

    ibs_Style        Meaning
    -----            -------
    ibs_VertCaption  Give the window a vertical caption.
    ibs_HorzCaption  Give the window a horizontal caption.

  If an invalid ibs_Style bit is specified, the default will be
  ibs_HorzCaption.
}
  begin
    inherited Init(aParent,aTitle);
    if (ibs_Style <> ibs_HorzCaption) and (ibs_Style <> ibs_VertCaption) then
      ibs_Style := ibs_HorzCaption;
    with Attr do begin
      Style :=
        ibs_Style or
        ws_ThickFrame or
        ws_Border or
        ws_Popup or
        ws_SysMenu or
        ws_Visible;
    end;
  end { TItsyBitsWindow.Init };

procedure TItsyBitsWindow.DefWndProc;
{
  Overrides TWindow.DefWndProc.  Calls ibDefWindowProc for certain
  messages.  Other messages are passed on to DefWndProc.

  If you add any more message handling capabilities to ibDefWindowProc,
  you must also update this method.
}
  begin
    case Msg.Message of
      wm_SysChar,
      wm_SysKeyDown,
      wm_SysKeyUp,
      wm_KeyDown,
      wm_KeyUp,
      wm_GetMinMaxInfo,
      wm_Command,
      wm_NCCreate,
      wm_NCCalcSize,
      wm_NCHitTest,
      wm_NCLButtonDblClk,
      wm_NCLButtonDown,
      wm_NCActivate,
      wm_NCPaint : begin
        with Msg do
          Result := ibDefWindowProc(Receiver,Message,wParam,lParam);
      end
    else
      inherited DefWndProc(Msg);
    end;
  end { TItsyBitsWindow.DefWndProc };

function TItsyBitsWindow.GetClassName;
{
  Returns a class name of 'ItsyWnd';
}
  begin
    GetClassName := 'ItsyWnd';
  end { TItsyBitsWindow.GetClassName };

{-- Local Functions -----------------------------------------------------}

function CaptionXY : integer;
  begin
    CaptionXY := GetSystemMetrics(sm_CYCaption) div 2 + 4;
  end { CaptionXY };

function GetStyle(HWin : hwnd) : word;
  begin
    GetStyle := LoWord(GetWindowLong(HWin,gwl_Style));
  end { GetStyle };

function HasCaption(HWin : hwnd) : boolean;
  begin
    HasCaption := ((GetStyle(HWin) and ibs_VertCaption) <> 0) or
                  ((GetStyle(HWin) and ibs_HorzCaption) <> 0);
  end { HasCaption };

function TestWinStyle(HWin       : hwnd;
                      dwStyleBit : longint) : bool;
  begin
    TestWinStyle := (GetWindowLong(HWin,gwl_Style) and dwStyleBit) <> 0;
  end { TestWinStyle };

{$ifdef USE_EXTTEXTOUT}
procedure DrawFastRect(    DC   : hdc;
                       var lprc : TRect);
  begin
    ExtTextOut(DC,0,0,eto_Opaque,@lprc,nil,0,nil);
  end { DrawFastRect };
{$else}
procedure DrawFastRect(    DC   : hdc;
                       var lprc : TRect);
  var
    hbr : HBrush;
  begin
    hbr := CreateSolidBrush(GetBkColor(DC));
    hbr := SelectObject(DC,hbr);
    PatBlt(DC,lprc.left,lprc.top,lprc.right - lprc.left,lprc.bottom - lprc.top,PatCopy);
    hbr := SelectObject(DC,hbr);
    DeleteObject(hbr);
  end;
{$endif}

function ibGetCaptionRect(    HWin : hwnd;
                          var lprc : TRect) : bool;
{
  Calcluales the rectangle of the mini-caption in screen coords.
}
  begin
    if not HasCaption(HWin) then begin
      SetRectEmpty(lprc);
      ibGetCaptionRect := false;
      exit;
    end;
    GetWindowRect(HWin,lprc);
    {
      The window might have other non-client components like borders.
    }
    if TestWinStyle(HWin,ws_ThickFrame) then
      begin
        inc(lprc.left,GetSystemMetrics(sm_CXFrame));
        inc(lprc.top,GetSystemMetrics(sm_CYFrame));
        dec(lprc.right,GetSystemMetrics(sm_CXFrame));
        dec(lprc.bottom,GetSystemMetrics(sm_CYFrame));
      end
    else
      if TestWinStyle(HWin,ws_Border) then begin
        inc(lprc.left,GetSystemMetrics(sm_CXBorder));
        inc(lprc.top,GetSystemMetrics(sm_CYBorder));
        dec(lprc.right,GetSystemMetrics(sm_CXBorder));
        dec(lprc.bottom,GetSystemMetrics(sm_CYBorder));
      end;
    if TestWinStyle(HWin,ibs_VertCaption) then
      lprc.right := lprc.left + gcCaption
    else
      lprc.bottom := lprc.top + gcCaption;
    ibGetCaptionRect := true;
  end { ibGetCaptionRect };

function ibGetIconRect(    HWin : hwnd;
                       var lprc : TRect) : boolean;
{
  Calculates the rect of the icon in screen coordinates.
}
  begin
    ibGetCaptionRect(HWin,lprc);   { window coords }
    if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
      lprc.bottom := lprc.top + gcCaption
    else
      lprc.right := lprc.left + gcCaption;
    dec(lprc.bottom);
    dec(lprc.right);
    ibGetIconRect := true;
  end { ibGetIconRect };

procedure ibDrawSysMenu(DC      : hdc;
                        HWin    : hwnd;
                        fInvert : bool);
  var
    rcIcon,
    rcTemp,
    rc             : TRect;
    rgbIconFace,
    rgbWindowFrame : TColorRef;
    fDC            : bool;
  begin
    if (DC = 0) then
      begin
        fDC := true;
        DC := GetWindowDC(HWin);
      end
    else
      fDC := false;

    if (DC <> 0) then begin
      rgbIconFace := GetNearestColor(DC,RGBLtGray);
      rgbWindowFrame := GetSysColor(color_WindowFrame);
      ibGetIconRect(HWin,rcIcon);
      GetWindowRect(HWin,rc);
      OffsetRect(rcIcon,-rc.left,-rc.top);
      rcTemp := rcIcon;
      if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
        begin
          rc := rcIcon;     { separator line }
          inc(rc.bottom);
          rc.top := rc.bottom - 1;
        end
      else
        begin
          rc := rcIcon;     { separator line }
          inc(rc.right);
          rc.left := rc.right - 1;
        end;
      { Fill }
      SetBkColor(DC,rgbIconFace);
      DrawFastRect(DC,rcTemp);
      { Draw separator line }
      SetBkColor(DC,rgbWindowFrame);
      DrawFastRect(DC,rc);

      { Draw little doo-hickey }
      rcTemp.top := rcIcon.top + ((gcCaption-1) div 2);
      rcTemp.bottom := rcTemp.top + 3;
      rcTemp.left  := rcTemp.left + 3;
      rcTemp.right := rcTemp.right - 1;
      SetBkColor(DC,RGBGray);
      DrawFastRect(DC,rcTemp);
      rc := rcTemp;
      OffsetRect(rc,-1,-1);
      SetBkColor(DC,RGBBlack);
      DrawFastRect(DC,rc);
      InflateRect(rc,-1,-1);
      SetBkColor(DC,RGBWhite);
      DrawFastRect(DC,rc);

      if fInvert then
        InvertRect(DC,rcIcon);
      if fDC then
        ReleaseDC(HWin,DC);
    end;
  end { ibDrawSysMenu };

procedure SetupSystemMenu(HWin : hwnd;
                          Menu : hmenu);
{
  Enables/Disables the appropriate menu items on the
  menu passed for the window passed.
}
  var
    wMove,
    wSize,
    wMinBox,
    wMaxBox,
    wRestore : word;
  begin
    {
      Assume all should be grayed.
    }
    wSize := mf_Grayed;
    wMove := mf_Grayed;
    wMinBox := mf_Grayed;
    wMaxBox :=  mf_Grayed;
    wRestore := mf_Grayed;

    if (TestWinStyle(HWin,ws_MaximizeBox)) or (IsIconic(HWin)) then
      wMaxBox := mf_Enabled;
    if (TestWinStyle(HWin,ws_MinimizeBox)) then
      wMinBox := mf_Enabled;
    if (IsZoomed(HWin)) then
      wRestore := mf_Enabled;
    if (TestWinStyle(HWin,ws_ThickFrame)) and
        (not (IsIconic(HWin) or (IsZoomed(HWin)))) then
      wSize := mf_Enabled;
    if (not IsZoomed(HWin)) and
       (not IsIconic(HWin)) and
       TestWinStyle(HWin,ws_Caption) then
      wMove := mf_Enabled;

    EnableMenuItem(Menu,sc_Move,wMove);
    EnableMenuItem(Menu,sc_Size,wSize);
    EnableMenuItem(Menu,sc_Minimize,wMinBox);
    EnableMenuItem(Menu,sc_Maximize,wMaxBox);
    EnableMenuItem(Menu,sc_Restore,wRestore);
  end { SetupSystemMenu };

function ibDoMenu(HWin : hwnd) : boolean;
{
  Pops up the menu.
}
  var
    DC     : hdc;
    rcIcon,
    rc     : TRect;
    pt     : TPoint;
    Menu   : HMenu;
    dw     : longint;
    TempPt : TPoint;
  begin
    if not TestWinStyle(HWin,ws_SysMenu) then begin
      ibDoMenu := false;
      exit;
    end;
    DC := GetWindowDC(HWin);
    if DC <> 0 then begin
      {
        Invert the icon
      }
      ibDrawSysMenu(DC,HWin,true);

      {
        Pop up the menu
      }
      if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
        begin
          pt.x := -1;
          pt.y := 0;
        end
      else
        begin
          pt.x := 0;
          pt.y := -1;
        end;

      ibGetIconRect(HWin,rcIcon);
      GetWindowRect(HWin,rc);
      OffsetRect(rcIcon,-rc.left,-rc.top);
      ClientToScreen(HWin,pt);
      TempPt.x := rc.right;
      TempPt.y := rc.bottom;
      ClientToScreen(HWin,TempPt);
      dw := GetWindowLong(HWin,gwl_Style);
      SetWindowLong(HWin,gwl_Style,dw or ws_SysMenu);
      Menu := GetSystemMenu(HWin,false);
      SetupSystemMenu(HWin,Menu);
      SetWindowLong(HWin,gwl_Style,dw);
      TrackPopupMenu(Menu, 0, { TPM_LEFTALIGN, }
                        pt.x,
                        pt.y,
                        0,
                        HWin,
                        @rc);
      ibDrawSysMenu(DC,HWin,false);
      ReleaseDC(HWin,DC);
    end;
    ibDoMenu := true;
  end { ibDoMenu };

function ibDrawCaption(    DC       : hdc;
                           HWin     : hwnd;
                       var lprc     : TRect;
                           fVert,
                           fSysMenu,
                           fActive  : bool) : bool;
{
  This function draws an itsy bitsy caption bar with or without system
  menu to the dc specified by hDC.  The caption is drawn to fit within
  the lprc RECT and is drawn *withOut* borders.
}
  var
    rc,
    rcCap          : TRect;
    rgbCaptionBG,
    rgbText,
    rgbWindowFrame : TColorRef;
    hbrCaption     : HBrush;
    ui             : word;
    gh             : THandle;
    Font           : hFont;
    lpsz           : pchar;
    plf            : PLogFont;
    tm             : TTextMetric;
    cx,
    cy,
    X,
    Y              : integer;
    dwTextExt      : longint;
  begin
    { Get the colors. }
    rgbWindowFrame := GetSysColor(color_WindowFrame);

    {
      If we have focus use the active caption color
      otherwise use the inactive caption color
    }
    if (fActive) then
      begin
        rgbText := GetSysColor(color_CaptionText);
        rgbCaptionBG := GetSysColor(color_ActiveCaption);
      end
    else
      begin
        rgbText := GetSysColor(color_InactiveCaptionText);
        rgbCaptionBG := GetSysColor(color_InactiveCaption);
      end;
    {
      Fill the caption color
    }
    hbrCaption := CreateSolidBrush(rgbCaptionBG);
    hbrCaption := SelectObject(DC,hbrCaption);
    SelectObject(DC,GetStockObject(NULL_PEN));
    rcCap := lprc;
    if (fSysMenu) then
      if (fVert) then
        inc(rcCap.top,gcCaption)
      else
        inc(rcCap.left,gcCaption);
    Rectangle(DC,rcCap.left,rcCap.top,rcCap.right+1,rcCap.bottom+1);

    if (fVert) then
      begin
        rc.right := lprc.right;
        rc.left := rc.right - 1;
        rc.top := lprc.top;
        rc.bottom := lprc.bottom;
      end
    else
      begin
        rc.left := lprc.left;
        rc.right := lprc.right;
        rc.bottom := lprc.bottom;
        rc.top := rc.bottom - 1;
      end;

    SetBkColor(DC,rgbWindowFrame);
    DrawFastRect(DC,rc);

    {
      Draw caption text here.  Only do it in 3.1 'cause 3.1 gives
      us 'small fonts'.
    }
    ui := GetWindowTextLength(HWin);
    gh := GlobalAlloc(GHND,ui + 2);
    if (fWin31) and (gh <> 0) then begin
      getmem(plf,sizeof(TLogFont));
      if (plf <> nil) then begin
        fillchar(plf^,sizeof(TLogFont),0);
        lpsz := GlobalLock(gh);
        GetWindowText(HWin,lpsz,ui + 1);

        {
          Setup the DC
        }
        SetBkMode(DC,TRANSPARENT);
        rgbText := SetTextColor(DC,rgbText);

        if (fVert) then
          begin
            {
              Can only rotate true type fonts (well, ok, we could
              try and use "modern".
            }
            lstrcpy(plf^.lfFaceName,'Arial');
            plf^.lfHeight := -(gcCaption - 3);
            plf^.lfEscapement := 900;
            plf^.lfQuality := PROOF_QUALITY;
            { plf^.lfOutPrecision := STROKE_PRECIS; }
            Font := CreateFontIndirect(plf^);
            Font := SelectObject(DC,Font);
            dwTextExt := GetTextExtent(DC,lpsz,ui);
            cx := rcCap.bottom - ((rcCap.bottom - rcCap.top -
                  LoWord(dwTextExt)) div 2);
            cy := rcCap.left + ((rcCap.right - rcCap.left -
                  HiWord(dwTextExt)) div 2) ;

            {
              Make sure we got a rotatable font back.
            }
            GetTextMetrics(DC,tm);
            if (tm.tmPitchAndFamily and TMPF_VECTOR) <> 0 then begin
              if cx < rcCap.bottom then
                Y := cx
              else
                Y := rcCap.bottom;
              ExtTextOut(DC,rcCap.left,Y,ETO_CLIPPED,@rcCap,lpsz,ui,nil);
            end;
            Font := SelectObject(DC,Font);
            DeleteObject(Font);
          end
        else
          begin
            {
              Use small fonts always for the horizontal. Cause it looks
              more like "System" than Arial.
            }
            plf^.lfHeight := -(gcCaption - 3);
            lstrcpy(plf^.lfFaceName,'Arial');
            plf^.lfQuality := PROOF_QUALITY ;
            Font := CreateFontIndirect(plf^);
            Font := SelectObject(DC,Font);
            dwTextExt := GetTextExtent(DC,lpsz,ui);
            cx := rcCap.left + ((rcCap.right - rcCap.left -
                  LOWORD(dwTextExt)) div 2);

            cy := rcCap.top - 1 + ((rcCap.bottom - rcCap.top -
                  HIWORD(dwTextExt)) div 2) ;

            {
              Figure out how big the string is.
            }
            if cx > rcCap.left then
              X := cx
            else
              X := rcCap.left;
            ExtTextOut(DC,X,cy,ETO_CLIPPED,@rcCap,lpsz,ui,nil);

            Font := SelectObject(DC,Font);
            DeleteObject(Font);
          end;

        {
          Un setup the DC
        }
        rgbText := SetTextColor(DC,rgbText);

        freemem(plf,sizeof(TLogFont));
        GlobalUnlock(gh);
      end;
      GlobalFree(gh);
    end;

    if (fSysMenu) then
      ibDrawSysMenu(DC,HWin,false);

    hbrCaption := SelectObject(DC,hbrCaption);
    DeleteObject(hbrCaption);

    ibDrawCaption := true;
  end { ibDrawCaption };

{-- Exported Functions --------------------------------------------------}

function ibDefWindowProc;
  var
    dwRet,
    dw      : longint;
    PMMI    : PMinMaxInfo;
    rc,
    rcCap   : TRect;
    pt      : TPoint;
    DC      : hdc;
    fActive : bool;
  begin
    case wMsg of
      wm_SysChar : begin
        {
          If ALT-SPACE was hit then pop up the menu.
        }
        if (HasCaption(HWin)) and (wParam = vk_Space) then
          ibDoMenu(HWin)
        else
          begin
            dw := GetWindowLong(HWin,gwl_Style);
            {
              Fool DefWindowProc.
            }
            SetWindowLong(HWin,gwl_Style,dw and not ws_SysMenu);
            dwRet := DefWindowProc(HWin,wMsg,wParam,lParam);
            SetWindowLong(HWin,gwl_Style,dw);
            ibDefWindowProc := dwRet;
            exit;
          end;
      end;
      wm_SysKeyDown,
      wm_SysKeyUp,
      wm_KeyDown,
      wm_KeyUp : begin
        dw := GetWindowLong(HWin,gwl_Style);
        {
          Fool DefWindowProc.
        }
        SetWindowLong(HWin,gwl_Style,dw and not ws_SysMenu);
        dwRet := DefWindowProc(HWin,wMsg,wParam,lParam);
        SetWindowLong(HWin,gwl_Style,dw);
        ibDefWindowProc := dwRet;
        exit;
      end;
      wm_GetMinMaxInfo : begin
        if (HasCaption(HWin)) and (TestWinStyle(HWin,ws_ThickFrame)) then begin
          PMMI := pointer(lParam);
          if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
            begin
              PMMI^.ptMinTrackSize.x := gcCaption +
                                        (GetSystemMetrics(sm_CXFrame) * 2) - 1;
              PMMI^.ptMinTrackSize.y := GetSystemMetrics(sm_CXMinTrack) div 2;
            end
          else
            begin
              PMMI^.ptMinTrackSize.y := gcCaption +
                                        (GetSystemMetrics(sm_CYFrame) * 2) - 1;
              PMMI^.ptMinTrackSize.x := GetSystemMetrics(sm_CXMinTrack) div 2;
            end;
        end;
        ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
        exit;
      end;
      wm_Command : begin
        {
          The menu that is poped up for the system menu with
          TrackPopupMenu() sends it's notifications as WM_COMMAND
          messages.  We need to translate these into
          WM_SYSCOMMAND messages.  All standard WM_SYSCOMMAND
          ids are greater than 0xF000.

          This could be a possible cause of confusion if the
          itsybitsy window had children that used SC_MOVE or SC_CLOSE
          as their IDs.  Take note and be careful.

          Also, because ibDefWindowProc looks at WM_COMMAND messages,
          you will need to be careful to call ibDefWindowProc() for
          any wm_command messages that you would normally ignore.
          Otherwise the system menu won't work.
        }
        if wParam >= $F000 then
          SendMessage(HWin,wm_SysCommand,wParam,lParam);
        ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
        exit;
      end;
      {
        Non-client area messages.  These are used to allow the
        minature caption bar to be handled correctly.
      }
      wm_NCCreate : begin
        {
          CAPTIONXY is a macro that calls GetSystemMetrics.
          gcCaption is a global variable.
        }
        gcCaption := CaptionXY;
        {
          Are we in 3.1?  If so we have some neat features
          we can use.
        }
        fWin31 := (LoWord(GetVersion) >= $0A03);
        ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam );
        exit;
      end;
      wm_NCCalcSize : begin
        {
          This is sent when the window manager wants to find out
          how big our client area is to be.  If we have a mini-caption
          then we trap this message and calculate the cleint area rect,
          which is the client area rect calculated by DefWindowProc()
          minus the width/height of the mini-caption bar
        }
        dwRet := DefWindowProc(HWin,wMsg,wParam,lParam);
        if HasCaption(HWin) then
          if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
            inc(PRect(lParam)^.left,gcCaption)
          else
            inc(PRect(lParam)^.top,gcCaption);
        ibDefWindowProc := dwRet;
        exit;
      end;
      wm_NCHitTest : begin
        {
          This message is sent whenever the mouse moves over us.
          We will depend on DefWindowProc for everything unless
          there is a mini-caption, in which case we will
          return HTCAPTION or HTSYSMENU.  When the user clicks
          or double clicks, NC_LBUTTON* message are sent with
          wParam equal to what we return here.
          This means that this is an ideal place to figure out
          where we are!
        }
        {
          Let defwindowproc handle the standard borders etc...
        }
        dwRet := DefWindowProc(HWin,wMsg,wParam,lParam);

        if (HasCaption(HWin)) and (dwRet = HTNOWHERE) then begin
          {
            If DefWindowProc returned HTCAPTION then we have to
            refine the area and return HTSYSMENU if appropriate.
          }
          pt.x := LoWord(lParam);
          pt.y := HiWord(lParam);
          ibGetCaptionRect(HWin,rc);   { window coords }
          if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
            begin
              rc.bottom := rc.top + gcCaption - 1;
              if (TestWinStyle(HWin,ws_SysMenu)) and (PtInRect(rc,pt)) then
                dwRet := HTSYSMENU
              else
                dwRet := HTCAPTION;
            end
          else
            begin
              rc.right := rc.left + gcCaption - 1;
              if (TestWinStyle(HWin,ws_SysMenu)) and (PtInRect(rc,pt)) then
                dwRet := HTSYSMENU
              else
                dwRet := HTCAPTION;
            end;
        end;
        ibDefWindowProc := dwRet;
        exit;
      end;
      wm_NCLButtonDblClk : begin
        if (HasCaption(HWin)) and (wParam = HTSYSMENU) then
          SendMessage(HWin,wm_Close,0,0)
        else
          begin
            ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
            exit;
          end;
      end;
      wm_NCLButtonDown : begin
        if (not HasCaption(HWin)) then begin
          ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
          exit;
        end;
        if (wParam = HTSYSMENU) then
          ibDoMenu(HWin)
        else
          begin
            ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
            exit;
          end;
      end;
      wm_NCActivate,
      wm_NCPaint : begin
        {
          Paint the non-client area here.  We will call DefWindowProc
          after we are done so it can paint the borders and so
          forth...
        }
        if (HasCaption(HWin)) then begin
          DC := GetWindowDC(HWin);
          ibGetCaptionRect(HWin,rcCap);
          GetWindowRect(HWin,rc);
          OffsetRect(rcCap,-rc.left,-rc.top);
          if (wMsg = WM_NCPAINT) then
            fActive := (HWin = GetActiveWindow)
          else
            fActive := bool(wParam);

          if (GetStyle(HWin) and ibs_VertCaption) <> 0 then
            ibDrawCaption(DC,HWIn,rcCap,true,
                          TestWinStyle(HWin,ws_SysMenu),
                          fActive)
          else
            ibDrawCaption(DC,HWin,rcCap,false,
                          TestWinStyle(HWin,ws_SysMenu),
                          fActive);
          ReleaseDC(HWin,DC);
        end;
        ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
        exit;
      end;
    else
      begin
        ibDefWindowProc := DefWindowProc(HWin,wMsg,wParam,lParam);
        exit;
      end;
    end;
    ibDefWindowProc := 0;
  end { ibDefWindowProc };

procedure ibAdjustWindowRect;
  var
    cx,
    cy : word;
  begin
    cx := 0;
    cy := 0;
    {
      First check Windows's styles, then our own.
    }
    if TestWinStyle(HWin,ws_ThickFrame) then
      begin
        cx := GetSystemMetrics(sm_CXFrame);
        cy := GetSystemMetrics(sm_CYFrame);
      end
    else
      if TestWinStyle(HWin,ws_Border) then
        begin
          cx := GetSystemMetrics(sm_CXBorder);
          cy := GetSystemMetrics(sm_CYBorder);
        end;
    InflateRect(lprc,cx,cy);
    if TestWinStyle(HWin,ibs_VertCaption) then
      dec(lprc.left,gcCaption)
    else
      if TestWinStyle(HWin,ibs_HorzCaption) then
        dec(lprc.top,gcCaption);
  end { ibAdjustWindowRect };

end { ItsyBits }.
